@@ -52,12 +52,13 @@ def start_tcp_lang_server(bind_addr, port, check_parent_process, handler_class):
     if not issubclass(handler_class, PythonLanguageServer):
         raise ValueError('Handler class must be an instance of PythonLanguageServer')
 
-    def shutdown_server(*args):
+    def shutdown_server(check_parent_process, *args):
         # pylint: disable=unused-argument
-        log.debug('Shutting down server')
-        # Shutdown call must be done on a thread, to prevent deadlocks
-        stop_thread = threading.Thread(target=server.shutdown)
-        stop_thread.start()
+        if check_parent_process:
+            log.debug('Shutting down server')
+            # Shutdown call must be done on a thread, to prevent deadlocks
+            stop_thread = threading.Thread(target=server.shutdown)
+            stop_thread.start()
 
     # Construct a custom wrapper class around the user's handler_class
     wrapper_class = type(
@@ -65,7 +66,7 @@ def start_tcp_lang_server(bind_addr, port, check_parent_process, handler_class):
         (_StreamHandlerWrapper,),
         {'DELEGATE_CLASS': partial(handler_class,
                                    check_parent_process=check_parent_process),
-         'SHUTDOWN_CALL': shutdown_server}
+         'SHUTDOWN_CALL': partial(shutdown_server, check_parent_process)}
     )
 
     server = socketserver.TCPServer((bind_addr, port), wrapper_class, bind_and_activate=False)
@@ -227,7 +228,7 @@ class PythonLanguageServer(MethodDispatcher):
         return {'capabilities': self.capabilities()}
 
     def m_initialized(self, **_kwargs):
-        pass
+        self._hook('pyls_initialized')
 
     def code_actions(self, doc_uri, range, context):
         return flatten(self._hook('pyls_code_actions', doc_uri, range=range, context=context))
@@ -286,7 +287,7 @@ class PythonLanguageServer(MethodDispatcher):
         return self._hook('pyls_signature_help', doc_uri, position=position)
 
     def folding(self, doc_uri):
-        return self._hook('pyls_folding_range', doc_uri)
+        return flatten(self._hook('pyls_folding_range', doc_uri))
 
     def m_text_document__did_close(self, textDocument=None, **_kwargs):
         workspace = self._match_uri_to_workspace(textDocument['uri'])
@@ -357,18 +358,50 @@ class PythonLanguageServer(MethodDispatcher):
         self.config.update((settings or {}).get('pyls', {}))
         for workspace_uri in self.workspaces:
             workspace = self.workspaces[workspace_uri]
-            workspace.update_config(self.config)
+            workspace.update_config(settings)
             for doc_uri in workspace.documents:
                 self.lint(doc_uri, is_saved=False)
 
-    def m_workspace__did_change_workspace_folders(self, added=None, removed=None, **_kwargs):
+    def m_workspace__did_change_workspace_folders(self, event=None, **_kwargs):  # pylint: disable=too-many-locals
+        if event is None:
+            return
+        added = event.get('added', [])
+        removed = event.get('removed', [])
+
         for removed_info in removed:
-            removed_uri = removed_info['uri']
-            self.workspaces.pop(removed_uri)
+            if 'uri' in removed_info:
+                removed_uri = removed_info['uri']
+                self.workspaces.pop(removed_uri, None)
 
         for added_info in added:
-            added_uri = added_info['uri']
-            self.workspaces[added_uri] = Workspace(added_uri, self._endpoint, self.config)
+            if 'uri' in added_info:
+                added_uri = added_info['uri']
+                workspace_config = config.Config(
+                    added_uri, self.config._init_opts,
+                    self.config._process_id, self.config._capabilities)
+                workspace_config.update(self.config._settings)
+                self.workspaces[added_uri] = Workspace(
+                    added_uri, self._endpoint, workspace_config)
+
+        root_workspace_removed = any(removed_info['uri'] == self.root_uri for removed_info in removed)
+        workspace_added = len(added) > 0 and 'uri' in added[0]
+        if root_workspace_removed and workspace_added:
+            added_uri = added[0]['uri']
+            self.root_uri = added_uri
+            new_root_workspace = self.workspaces[added_uri]
+            self.config = new_root_workspace._config
+            self.workspace = new_root_workspace
+        elif root_workspace_removed:
+            # NOTE: Removing the root workspace can only happen when the server
+            # is closed, thus the else condition of this if can never happen.
+            if self.workspaces:
+                log.debug('Root workspace deleted!')
+                available_workspaces = sorted(self.workspaces)
+                first_workspace = available_workspaces[0]
+                new_root_workspace = self.workspaces[first_workspace]
+                self.root_uri = first_workspace
+                self.config = new_root_workspace._config
+                self.workspace = new_root_workspace
 
         # Migrate documents that are on the root workspace and have a better
         # match now
